/*  Canon Bubble Jet Print Filter.
 *  Copyright CANON INC. 2001-2002
 *  All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include	<signal.h>
#include	<sys/types.h>
#include	<unistd.h>
#include	<stdio.h>
#include	<fcntl.h>
#include	<stdlib.h>
#include	<asm/errno.h>
#include	<sys/ioctl.h>
#include	<linux/lp.h>
#ifdef DATA_LOG
#include	<sys/stat.h>
#endif

#include	"lmsmsock.h"
#include	"lm.h"

volatile static int end_signal   = 0; /* SIGTERM = program end       */
volatile static int last_signal  = 0; /* SIGUSR1 = print data is end */
volatile static int start_signal = 0; /* SIGUSR2 = data write start  */
extern int sem_id;
extern int errno;
extern int device_type;

#define DETECT_LAST  0x01 /* SIGUSR1 detect */
#define DETECT_END   0x02 /* SIGTERM detect */
#define DETECT_START 0x04 /* SIGUSR2 detect */
#define DETECT_NOT   0    /* no signals */

#define signal_last(x)  ((x&DETECT_LAST) == DETECT_LAST)
#define signal_end(x)   ((x&DETECT_END) == DETECT_END)
#define signal_start(x) ((x&DETECT_START) == DETECT_START)

#ifdef DEBUG
extern FILE *log_path;
#endif

static int check_signals(void)
{
	int sig = 0;

	if(end_signal){
#ifdef DEBUG
		write_log("LM(P) SIGTERM receive(data_output_process)\n");
#endif
		sig |= DETECT_END; /* SIGTERM */
	}

	if(last_signal){
#ifdef DEBUG
		write_log("LM(P) SIGUSR1 receive(data_output_process)\n");
#endif
		/* data read until pipe empty */
		fcntl(DATA_PATH, F_SETFL, fcntl(DATA_PATH, F_GETFL, 0) | O_NONBLOCK);
		last_signal = 0;
		sig |= DETECT_LAST; /* SIGUSR1 */
	}

	if(start_signal){
#ifdef DEBUG
		write_log("LM(P) SIGUSR2 receive(data_output_process)\n");
#endif
		sig |= DETECT_START;
	}

	return sig;
}

static int read_data(char *buf)
{
	int count;
	int sig = 0;

	while(1){
		count = read(DATA_PATH , buf, MAX_DATA);
		if(count < 0){
#ifdef DEBUG
			write_log("LM(P) read error\n");
#endif
			/* read error !? and check */
			sig = check_signals();
			if(signal_last(sig))
				continue; /* retry */
			if(signal_start(sig)){
				start_signal = 0;
				continue;
			}

			return -1; /* SIGTERM or EAGAIN or other read error */
		}

		break; /* read SUCCESS */
	}

	return count;
}

void lm_dataout_sighand(int sigcode)
{
	/* SIGTERM receive */
	end_signal = 1;
}

void lm_last_sighand(int sigcode)
{
	/* SIGUSR1 receive */
	last_signal = 1;
}

void lm_start_sighand(int sigcode)
{
	/* SIGUSR2 receive */
	start_signal = 1;
}

int wait_restart(void)
{
	int sig;

	/* wait restart(BSCC success) */
	while(1){
		sig = check_signals();

		if(signal_end(sig))
			return DETECT_END; /* SIGTERM */

		if(signal_start(sig)){
			start_signal = 0;
			return DETECT_START;
		}

		pause(); /* wait signal */
	}
}

int check_power(int sm_sock_fd, int dev_path)
{
	int flt;

	status_to_sm(sm_sock_fd, dev_path, &flt);

	return flt;
}


int data_output_process(int sm_sock_fd)
{
	int sig = 0;
	int error;
	int r_size;
	int w_size;
	int dev_path;
	char *buf;
	char *ptr;

#ifdef DATA_LOG
	int log_hand;

	log_hand = open("/var/log/bjfilter-data.log", O_WRONLY|O_CREAT|O_TRUNC, 0666);
#endif

#ifdef DEBUG
	write_log("LM(P) data_output_process start\n");
#endif

	/* setup signal handler */
	signal_setup(SIGTERM, lm_dataout_sighand);
	signal_setup(SIGUSR1, lm_last_sighand);
	signal_setup(SIGUSR2, lm_start_sighand);

	/* wait PRT_START */
	while(1){
/*
		if(end_signal || last_signal){
*/
		if(end_signal){
#ifdef DEBUG
			write_log("LM(P) SIGTERM receive\n");
			write_log("LM(P) data_output_process end\n");
#endif
			exit(0);
		}
		if(start_signal){
			start_signal = 0;
			break;
		}

		pause();
	}

	/* read buffer get */
	buf = (char *)malloc(MAX_DATA);
	if(!buf)
		exit(0);

#ifdef DEBUG
	write_log("LM(P) print start!\n");
#endif

	dev_path = dup(PRNT_PATH);
	close(PRNT_PATH);

	/* print data read(from stdin) and write(to stdout) */
	while((r_size = read_data(buf)) > 0){

		ptr = buf;
		/* data print(output) loop */
		for(;r_size > 0;r_size -= w_size){
			int err;

			/* printer lock start */
			error = get_printer_sem(sem_id);

			/* semapho get error ? */
			if(error){
				if((check_signals() & DETECT_END) == DETECT_END)
					goto dataout_exit; /* print end */

				w_size = 0;
				continue; /* retry */
			}

			w_size = write(dev_path, ptr, r_size); /* data write -> printer */
			err = errno;
#ifdef DATA_LOG
			if(log_hand >=0 && w_size > 0)
				write(log_hand, ptr, w_size);
#endif

			sig = check_signals();
			if(sig){
				release_printer_sem(sem_id);

				if(signal_end(sig))
					goto dataout_exit; /* SIGTERM: program end */

				/* SIGUSR1/SIGUSR2 */
				start_signal = 0;
				if(w_size <= 0){
					/* error reason = signal */
					w_size = 0;
					continue; /* retry */
				}
				/* write() is success */
#ifdef DEBUG
				fprintf(log_path, "LM(P) sig detect but write success\n");
#endif
				ptr += w_size;
				continue;
			}

			/* write() error check (power off??) */
			if(w_size <= 0){
				w_size = 0;
				if(check_power(sm_sock_fd, dev_path) == LM_PRN_POWOFF){
#ifdef DEBUG
					write_log("LM(P) into wait_restart()\n");
#endif
					release_printer_sem(sem_id);

					/* wait BSCC SUCCESS */
					if(wait_restart() == DETECT_END)
						goto dataout_exit;
					else
						continue;
				}
				release_printer_sem(sem_id);
#ifdef DEBUG
				fprintf(log_path, "LM(P) write error. normal=%x\n",err);
#endif
				sleep(1);
				continue;
			}
			/* write() is normal end */
			release_printer_sem(sem_id);
			ptr += w_size;
		}
	}

dataout_exit:
	if(end_signal == 0){
#ifdef DEBUG
		write_log("LM(P) data output loop is end. now wait SIGTERM\n");
#endif
		/* wait SIGTERM from langmon.c */
		while(1){
			if(end_signal)
				break;
			pause();
		}
	}
	free(buf);

#ifdef DEBUG
	write_log("LM(P) data_output_process end\n");
#endif

	close(dev_path);

#ifdef DATA_LOG
	if(log_hand >=0)
		close(log_hand);
#endif

	exit(0);
}

