/*  Canon Bubble Jet Print Filter.
 *  Copyright CANON INC. 2001-2002
 *  All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include	<signal.h>
#include	<sys/types.h>
#include	<sys/socket.h>
#include	<unistd.h>
#include	<stdio.h>
#include	<sys/wait.h>
#include	<fcntl.h>
#include	<sys/ioctl.h>
#include	<linux/lp.h>
#include	<sys/poll.h>
#include	<asm/errno.h>

#include	"lm.h"
#include	"lmsmsock.h"

volatile static int abort_signal = 0;
extern int sem_id;
extern int device_type;
extern int errno;

#ifdef DEBUG
extern FILE *log_path;
#endif

int status_get_main(int, int);
int fault_check_process(int);

void lm_stget_sighand(int sig)
{
	/* SIGTERM receive */
	abort_signal = 1;
}

void wait_readtime()
{
	if(device_type == DEVICE_TYPE_USB)
		sleep(2);
	else
		sleep(5);
	return;
}


int status_get_process(int sm_sock_fd)
{
	int dev_path;

	/* signal handler setup */
	signal_setup(SIGTERM, lm_stget_sighand);

#ifdef DEBUG
	write_log("LM(S) status_get_process start\n");
#endif

	dev_path = dup(PRNT_PATH);
	close(PRNT_PATH);

	/* printer status read until SIGTERM receive */
	while(1){

/*
		wait_readtime();
*/

		/* SIGTERM receive? */
		if(abort_signal){
#ifdef DEBUG
			write_log("LM(S) SIGTERM receive(status_get_process)\n");
#endif
			break;
		}

		status_get_main(sm_sock_fd, dev_path);

		/* SIGTERM receive? */
		if(abort_signal){
#ifdef DEBUG
			write_log("LM(S) SIGTERM receive(status_get_process)\n");
#endif
			break;
		}

		wait_readtime();
	}

#ifdef DEBUG
	write_log("LM(S) status_get_process end\n");
#endif

	close(dev_path);
	exit(0);
}

int status_get_main(int sm_sock_fd, int dev_path)
{
	int error;
	int flt;

	error = get_printer_sem(sem_id);

	/* semapho get error maybe SIGTERM received..... */
	if(error)
		return -1;

	status_to_sm(sm_sock_fd, dev_path, &flt);

	release_printer_sem(sem_id);

	return 0;
}

