/*  Canon Bubble Jet Print Filter.
 *  Copyright CANON INC. 2001-2002
 *  All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include	<arpa/inet.h>
#include	<netinet/in.h>
#include	<unistd.h>
#include	<sys/socket.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<sys/ipc.h>
#include	<sys/sem.h>
#include	<linux/lp.h>
#include	<stdio.h>
#include	<string.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<signal.h>
#include	<sys/ioctl.h>
#include	<stdlib.h>
#include	<asm/errno.h>

#include	"lmsmsock.h"
#include	"lm.h"

/* semapho union define */
union semun
{
	int val;                           /* value for SETVAL */
	struct semid_ds *buf;              /* buffer for IPC_STAT & IPC_SET */
	unsigned short int *array;         /* array for GETALL & SETALL */
	struct seminfo *__buf;             /* buffer for IPC_INFO */
};

extern int sem_id;
extern char *stamon_arg[ARGV_MAX];
extern int filter_exist;
extern int device_type;
extern int errno;

#ifdef DEBUG
FILE *log_path;
#endif

int create_sem(int *);
int create_arg(int, char **);

int get_printer_devid()
{
	return 0;
}

int lm_init(int dev_path)
{
	int error;
	struct stat st;

	/* printer access semapho create */
	error = create_sem(&sem_id);
	if(error)
		return error;

        fstat(dev_path, &st);
        device_type = (st.st_rdev&0x00ff00)>>8;
#ifdef DEBUG
        if(device_type == DEVICE_TYPE_LP)
                write_log("device = LP\n");
        else if(device_type == DEVICE_TYPE_USB)
                write_log("device = USB\n");
        else
                write_log("device = unknown\n");
#endif

	/* parport setup */
	if(device_type == DEVICE_TYPE_LP){
		get_printer_sem(sem_id);
		ioctl(dev_path, LPABORT, 1); /* if error -> abort read/write */
		fcntl(dev_path, F_SETFL, fcntl(dev_path, F_GETFL, 0) | O_NONBLOCK); /* Non Blocking IO select */
		release_printer_sem(sem_id);
	}

	return 0;
}

int create_sem(int *sem)
{
	key_t ipckey;
	union semun sem_union;
	int s;

	ipckey = IPC_PRIVATE;

	s = semget(ipckey, 1, 0666|IPC_CREAT);
	if(s == -1){
#ifdef DEBUG
		write_log("semget error\n");
#endif
		return -1;
	}

	/* init semapho=1 */
	sem_union.val = 1;
	if(semctl(s, 0, SETVAL, sem_union) == -1){
		/* error */
		remove_sem(s);
		return -1;
	}

	/* save semapho id */
	*sem = s;

	return 0;
}

int remove_sem(int sem)
{
	union semun sem_union;

	return semctl(sem, 0, IPC_RMID, sem_union);
}

int get_printer_sem(int id)
{
	struct sembuf sem_buf;

	if(device_type == DEVICE_TYPE_USB)
		return 0;

	sem_buf.sem_num = 0;
	sem_buf.sem_op  = -1;
	sem_buf.sem_flg = SEM_UNDO;

	while(1){
		if(semop(id, &sem_buf, 1) == -1)
			continue;
		else
			break;
	}

/* 2002-02-26 removed.
	if(semop(id, &sem_buf, 1) == -1){
		return -1;
	}
*/

	return 0;	
}

int release_printer_sem(int id)
{
	struct sembuf sem_buf;

	if(device_type == DEVICE_TYPE_USB)
		return 0;

	sem_buf.sem_num = 0;
	sem_buf.sem_op  = 1;
	sem_buf.sem_flg = SEM_UNDO;

	while(1){
		if(semop(id, &sem_buf, 1) == -1)
			continue;
		else
			break;
	}

/* 2002-02-26 removed.
	if(semop(id, &sem_buf, 1) == -1){
		return -1;
	}
*/

	return 0;
}

int lm_create_server_socket()
{
	struct sockaddr_in sin;
	int server_fd;
	int i;

	if((server_fd = socket(PF_INET, SOCK_STREAM, 0)) < 0)
		return -1;

	i = 1;
	setsockopt(server_fd, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));

	sin.sin_family = AF_INET;
	sin.sin_port = htons(SM_COM_PORT);
	memset(&sin.sin_addr, 0, sizeof(sin.sin_addr));

	if(bind(server_fd, (struct sockaddr *)&sin, sizeof(sin))){
		close(server_fd);
		return -1;
	}

	if(listen(server_fd, 5)){
		close(server_fd);
		return -1;
	}

	return server_fd;
}

int lm_wait_client_connect(int fd)
{
	struct sockaddr_in sin;
	size_t adrlen = sizeof(struct sockaddr_in);
	int client_fd;

	client_fd = accept(fd, (struct sockaddr *)&sin, &adrlen);

	return client_fd;
}

int write_printer_command(int dev_path, char *buf, int size)
{
	int w_size;

	if(!size)
		return 0;

	w_size = write(dev_path, buf, size);

	if(w_size == size)
		return 0;

	return -1;
}

int do_printer_reset(int dev_path)
{
	int error;

	if(device_type == DEVICE_TYPE_USB)
		return 0; /* USB driver(printer.o) is printer reset function no support */

	get_printer_sem(sem_id);

	error = ioctl(dev_path, LPRESET);

	release_printer_sem(sem_id);

	return error;
}

int get_printer_status(int dev_path, char *buf, int *reg)
{
	int error;
	int status;

	if((error = read(dev_path, (void *)buf, (size_t)MAX_STATBUF)) < 0){
#ifdef DEBUG
		fprintf(log_path, "status read error(R)=%x\n",errno);
#endif
		return errno;
	}

        if(device_type == DEVICE_TYPE_USB){
                *reg = PARPORT_NFAULT; /* set default(dummy) */
                return 0;
        }

	error = ioctl(dev_path, LPGETSTATUS, &status);
	if(!error){
		*reg = status;
		return 0;
	}

#ifdef DEBUG
	fprintf(log_path, "status read error(I)=%x\n",errno);
#endif

	return errno;
}

int check_arg(int ac, char **av)
{
	int i;
	int error = -1;

	if(ac == 1)
		return -1;

	for(i = 1;ac > i;i++){
		/* check gui flag */
		if((error == -1) && (strcmp(&(av[i][0]), "--gui") == 0)){
			/* create argv(stamon_arg) for status monitor */
			create_arg(ac, av);
			error = 0; /* "--gui" detect */
			continue;
		}
		/* check parent process */
		if(strcmp(&(av[i][0]), "--noparent") == 0)
			filter_exist = 0;
	}

	return error; 
}

int create_arg(int ac, char **av)
{
	int i;

	/* copy "/usr/local/bin/stsmon" string */
	stamon_arg[0] = strdup(STMON_PATH);

	/* copy argv(from bjfilter) */
	for(i = 1; ac > i; i++){
		stamon_arg[i] = strdup(av[i]);
	}

	stamon_arg[ac]   = NULL;

#ifdef DEBUG
	i = 0;
	while(1){
		write_log(stamon_arg[i++]);
		if(stamon_arg[i] == NULL)
			break;
		write_log(" ");
	}
	write_log("\n");
#endif

	return 0;
}

int free_arg()
{
	int i = 0;

	while(1){
		free(stamon_arg[i++]);
		if(stamon_arg[i] == NULL)
			break;
	}

	return 0;
}



#ifdef DEBUG
void create_log(void)
{
        FILE *path;

        path = fopen("/var/log/log.txt", "w");

        log_path = path;
        write_log("**********  LOG start  ***********\n");
}

void write_log(char *str)
{
	if(log_path == 0)
		return ;

        fwrite(str, 1, strlen(str), log_path);
        fflush(log_path);
}
#endif

int signal_setup(int sig, void (*func)())
{
	struct sigaction sa;

	memset(&sa, 0, sizeof(sa));
	sa.sa_handler = func;

	while(1){
		if(sigaction(sig, &sa, NULL) == 0)
			break;
#ifdef DEBUG
		write_log("sigaction error\n");
#endif
	}

	return 0;
}

int status_to_sm(int sock, int dev_path, int *flt)
{
	int error;
        char *buf;
        int reg = 0;
        lmsm_socket sm_data;
	int retry;
	int count;

	buf = (char *)malloc(MAX_STATBUF);
	if(buf == NULL)
		return 0; /* no RAM */

        memset(buf, 0, MAX_STATBUF);

        /* create socket data */
        sm_data.socket_type = LMtoSM;
        sm_data.command     = 0;
	sm_data.device_type = device_type; /* USB or PARPORT */

        /* printer status get */
	for(retry = 3; retry > 0; retry --){
        	error = get_printer_status(dev_path, buf, &reg);
		if(!error || error == EAGAIN)
			break;
	}

	switch(error){
		case 0:
			/* read SUCCESS */
			if(reg & PARPORT_NFAULT)
				sm_data.prn_data.stat.printer_fault = LM_PRN_NORMAL;
			else
				sm_data.prn_data.stat.printer_fault = LM_PRN_FAULT;
			break;
		case EAGAIN:
			/* no read data */
			sm_data.prn_data.stat.printer_fault = LM_PRN_NORMAL;
			break;
		default:
			/* read error */
			sm_data.prn_data.stat.printer_fault = LM_PRN_POWOFF; /* Power off */
#ifdef DEBUG
			write_log("Power off detect\n");
#endif
			break;
	}

	*flt = sm_data.prn_data.stat.printer_fault; /* save fault status */

	count = strlen(buf+2);
        memcpy(sm_data.prn_data.stat.stat_buffer, buf, MAX_STATBUF);

#ifdef DEBUG
	if(buf[2] != (char)NULL){
		write_log("STATUS=");
		write_log(buf+2);
		write_log("\n");
	}
#endif

        /* send socket -> SM */
	if(sock)
		write(sock, &sm_data, sizeof(sm_data));

	free(buf);

	return count;

}

